/****************************

    hotspot.c --
    
***************************/    
#include <windows.h>
#include <mmsystem.h>
#include <digitalv.h>
#include <viewer.h>
#include <string.h>
#include <direct.h>

#include "..\dll\hotspot.h"

// define VIEWER for use with VIEWER (i.e. for vwrCommand() )

char szMovieInfo[] = "MovieInfo";

/*********************************************************************************************
    FUNCTION:   InitHotspots(PMOVIEINFO, LPSTR)
    
    PURPOSE:    Reads specified Ini File.  Returns TRUE if successful, FALSE if not.
                Takes a pointer to a MOVIEINFO struct and the name of the ini file to read.
*********************************************************************************************/
BOOL InitHotspots(PMOVIEINFO pMovieInfo, LPSTR pszIniFile)
{

    char    szHotspots[1024];
    LPSTR   p;  
    char    szBuf[129];
    PHOTSPOT pHotspot;
    LPSTR   token;    
    char szDir[129];
    char szIniFile[129];
    
    
#ifdef VIEWER       // get the correct file...
    if (NULL == (_fstrchr(pszIniFile, '\\'))) // if no dir specified, make full path
        {        
        if (_getcwd(szDir, 128) == NULL)
            MessageBox(NULL, "Error getting current directory.", "Error", MB_OK);
        else
            {
            if (szDir[strlen(szDir) - 1] != '\\')
                strcat(szDir, "\\");
            wsprintf(szIniFile, "%s%s", szDir, pszIniFile);        
            }
        }
    else                                    // otherwise use the path we have
        lstrcpy(szIniFile, pszIniFile);                    
#else
    lstrcpy(szIniFile, pszIniFile);
#endif    

    if (!GetPrivateProfileString("Hotspots",
                            NULL,
                            "",
                            szHotspots,
                            sizeof(szHotspots),
                            szIniFile))
        {
        MessageBox(NULL, "Unable to load hotspots for AVI file.  Please check that"
                         " files are installed and the correct path is specified.",
                         "Error", MB_OK | MB_ICONEXCLAMATION);
        }                            
    p = szHotspots;
    while (*p) {        
        GetPrivateProfileString(p,
                            "Rect",
                            "",
                            szBuf,
                            sizeof(szBuf),
                            szIniFile);
        
        if (!lstrlen(szBuf)) {
            dprintf("Error: No Rect entry for hotspot");
        } else
            {
            if (! (pHotspot = (PHOTSPOT)ALLOCATE(sizeof(HOTSPOT))))
                {
                dprintf("Unable to allocate HOTSPOT structure");
                }
            else
                {                
                token = FindToken(szBuf, ',');
                pHotspot->rc.left = StrToInt(token);
                
                token = FindToken(NULL, ',');
                pHotspot->rc.top = StrToInt(token);
                
                token = FindToken(NULL, ',');
                pHotspot->rc.right = StrToInt(token);
                
                token = FindToken(NULL, ',');
                pHotspot->rc.bottom = StrToInt(token);
                
                GetPrivateProfileString(p,
                            "Command",
                            "",
                            szBuf,
                            sizeof(szBuf),
                            szIniFile);
                                                                    
                pHotspot->pszCommand = (LPSTR) ALLOCATE(lstrlen(szBuf) + 1);
                lstrcpy(pHotspot->pszCommand, szBuf);
                
                pHotspot->pszHotspotID = (LPSTR) ALLOCATE(lstrlen(p) + 1);
                lstrcpy(pHotspot->pszHotspotID, p);                
                
                pHotspot->BeginFrame = GetPrivateProfileInt(p,
                                                  "BeginFrame",
                                                  0,
                                                  szIniFile);
                                                  
                pHotspot->EndFrame = GetPrivateProfileInt(p,
                                                  "EndFrame",
                                                  0,
                                                  szIniFile);
                                                  
                pHotspot->OnClick = GetPrivateProfileInt(p,
                                                  "OnClick",
                                                  ID_CONTINUE,
                                                  szIniFile);
                                                  
                pHotspot->ToFrame = GetPrivateProfileInt(p,
                                                  "ToFrame",
                                                  0,
                                                  szIniFile);                                                                  
                                                  
                AddHotspot(pMovieInfo, pHotspot);
                }
            }
        p += lstrlen(p) + 1;
        }
    
    return TRUE;
}

/*********************************************************************************************
    FUNCTION:   DeleteHotspot(PMOVIEINFO, PHOTSPOT)
    
    PURPOSE:    Deletes specified hotspot from specified MovieInfo.
                No return value.
*********************************************************************************************/
void DeleteHotspot(PMOVIEINFO pMovieInfo, PHOTSPOT pHotspot)
{
    PHOTSPOT pPrev, pNext;    
    
    if (!pMovieInfo || !pHotspot)
        return;

    pPrev = pHotspot->pPrev;
    pNext = pHotspot->pNext;
    
    if (pPrev) {
        pPrev->pNext = pNext;
    } else {
        pMovieInfo->pHotspotList = pNext;
    }
    if (pNext) {
        pNext->pPrev = pPrev;
    }

    if (pHotspot->pszCommand)
        FREE(pHotspot->pszCommand);

    if (pHotspot->pszHotspotID)
        FREE(pHotspot->pszHotspotID);
                
    FREE(pHotspot);    
    pHotspot = NULL;
}

/*********************************************************************************************
    FUNCTION:   DeleteHotspotList(PMOVIEINFO)
    
    PURPOSE:    Deletes an entire hotspot list given a pointer to a MOVIEINFO struct.
*********************************************************************************************/
void DeleteHotspotList(PMOVIEINFO pMovieInfo)
{
    PHOTSPOT pHotspot, pNext;
    
    pHotspot = pMovieInfo->pHotspotList;
    while (pHotspot) {
        pNext = pHotspot->pNext;
        DeleteHotspot(pMovieInfo, pHotspot);
        pHotspot = pNext;
    }
    pMovieInfo->pHotspotList = NULL;
}

/*********************************************************************************************
    FUNCTION:   AddHotspot(PMOVIEINFO, PHOTSPOT)
    
    PURPOSE:    Adds specified hotspot to hotspot list in given MOVIEINFO.
                Requres a pointer to a MOVIEINFO struct and a pointer to a HOTSPOT struct
                allocated with ALLOCATE (see header file).
*********************************************************************************************/
void AddHotspot(PMOVIEINFO pMovieInfo, PHOTSPOT pHotspot)
{    

    if (pMovieInfo->pHotspotList) {
        pMovieInfo->pHotspotList->pPrev = pHotspot;
        pHotspot->pNext = pMovieInfo->pHotspotList;
    }
    pMovieInfo->pHotspotList = pHotspot;
}

