/*******

     movie.c --
     
     Functions for opening, playing, closing, setting position, getting current frame, etc.
     of a movie specified by a PMOVIEINFO paramater (a long ptr to a MOVIEINFO struct)
     
*******/     

#include <windows.h>
#include <mmsystem.h>
#include <digitalv.h>       
#include <viewer.h>

#include "..\dll\hotspot.h"

/*--------------------------------------------------------------+ 
| positionMovie - sets the movie rectange <rcMovie> to be   |
|       centered within the app's window.       |
|                               |
+--------------------------------------------------------------*/
VOID positionMovie(PMOVIEINFO pMovieInfo)
{
   RECT rcClient, rcMovieBounds;
   MCI_DGV_RECT_PARMS   mciRect;
   BOOL retcode = TRUE;
   
   /* if there is no movie yet then just get out of here */
   if (!pMovieInfo->fMovieOpen)
       return;
   
   GetClientRect(pMovieInfo->hwndParent, &rcClient);  /* get the parent windows rect */
       
   /* get the original size of the movie */
   mciSendCommand(pMovieInfo->wMCIDeviceID, MCI_WHERE, 
                  (DWORD)(MCI_DGV_WHERE_SOURCE), 
                  (DWORD)(LPMCI_DGV_RECT_PARMS)&mciRect);

#ifdef AVIHAPP                  
    if (pMovieInfo->lpfnhspAVICallback)
        {
        RECT FAR *rcFar;        
        
        rcFar = (RECT FAR *)ALLOCATE(sizeof (RECT));
        CopyRect(rcFar, &mciRect.rc);        
/**        
        pMovieInfo->lpfnhspAVICallback (pMovieInfo->hwndParent,
                                                pMovieInfo->hwndMovie,
                                                WM_SIZE, 
                                                (LONG)(RECT FAR *)rcFar, 
                                                (LONG)NULL);
        FREE(rcFar);                                                
        }                                                
***/        
#else        
                  
   rcMovieBounds = mciRect.rc;  /* get it in the movie bounds rect */
   
   pMovieInfo->rcMovie.left = (rcClient.right/2) - (rcMovieBounds.right / 2);
   pMovieInfo->rcMovie.top = (rcClient.bottom/2) - (rcMovieBounds.bottom / 2);
   pMovieInfo->rcMovie.right = pMovieInfo->rcMovie.left + rcMovieBounds.right;
   pMovieInfo->rcMovie.bottom = pMovieInfo->rcMovie.top + rcMovieBounds.bottom;
   
   /* reposition the playback (child) window */
   MoveWindow(pMovieInfo->hwndMovie, pMovieInfo->rcMovie.left, pMovieInfo->rcMovie.top,
       rcMovieBounds.right, rcMovieBounds.bottom, TRUE);
#endif       
}

/*--------------------------------------------------------------+ 
| fileCloseMovie - close the movie and anything associated  |
|          with it.                 |
|                               |
| This function clears the <fPlaying> and <fMovieOpen> flags    |
|                               |
+--------------------------------------------------------------*/
void fileCloseMovie(PMOVIEINFO pMovieInfo, BOOL bUpdate)
{
  MCI_GENERIC_PARMS  mciGeneric;
  
  pMovieInfo->fPlaying = FALSE; // can't be playing any longer
  pMovieInfo->fMovieOpen = FALSE;   // no more movies open
  
  mciSendCommand(pMovieInfo->wMCIDeviceID, MCI_CLOSE, MCI_WAIT, 
                 (DWORD)(LPMCI_GENERIC_PARMS)&mciGeneric);
                 
  if (bUpdate)
    {    
    /* cause a total repaint to occur */
    InvalidateRect(pMovieInfo->hwndParent, NULL, TRUE);
    UpdateWindow(pMovieInfo->hwndParent);
    }
}
/*******************************************************************************************
    FUNCTION:   GetMovieLength(PMOVIEINFO)
    
    PURPOSE:    returns length of specified movie
*******************************************************************************************/
DWORD GetMovieLength(PMOVIEINFO pMovieInfo)
{
    MCI_STATUS_PARMS mciStatus;

    mciStatus.dwTrack = 1;
    mciStatus.dwCallback = NULL;
    mciStatus.dwItem = MCI_STATUS_LENGTH;

    if (0 != (mciSendCommand(pMovieInfo->wMCIDeviceID, MCI_STATUS, (DWORD)MCI_STATUS_ITEM,
            (DWORD)(LPMCI_STATUS_PARMS)&mciStatus)))
        {
        dprintf("Error getting movie length");
        return 0;
        }
            
    return (mciStatus.dwReturn);
}

/*******************************************************************************************
    FUNCTION:   GetMovieFrame(PMOVIEINFO)
    
    PURPOSE:    returns current movie frame
*******************************************************************************************/
DWORD GetMovieFrame(PMOVIEINFO pMovieInfo)
{
    MCI_STATUS_PARMS mciStatus;

    mciStatus.dwTrack = 1;
    mciStatus.dwCallback = NULL;
    mciStatus.dwItem = MCI_STATUS_POSITION;

    if (0 != (mciSendCommand(pMovieInfo->wMCIDeviceID, MCI_STATUS, (DWORD)MCI_STATUS_ITEM,
            (DWORD)(LPMCI_STATUS_PARMS)&mciStatus)))
        {
        dprintf("Error getting movie frame");
        return 0;
        }
            
    return (mciStatus.dwReturn);            
}

/*--------------------------------------------------------------+ 
| fileOpenMovie - open an AVI movie. Use CommDlg open box to    |
|           open and then handle the initialization to  |
|       show the movie and position it properly.  Keep  |
|       the movie paused when opened.           |
|                               |
|       Sets <fMovieOpened> on success.         |
+--------------------------------------------------------------*/
void fileOpenMovie(PMOVIEINFO pMovieInfo, LPSTR szFilename)
{
   
    if (pMovieInfo == NULL)
        {
        dprintf("No MOVIEINFO for specified movie when trying to open AVI file.",
                "Error", MB_OK | MB_ICONHAND);
        return;
        }

   if (lstrlen(szFilename)){
     MCI_DGV_OPEN_PARMS mciOpen;
     MCI_DGV_WINDOW_PARMS   mciWindow;
     MCI_DGV_STATUS_PARMS   mciStatus;

     /* we got a filename, now close any old movie and open */
     /* the new one.                    */
     if (pMovieInfo->fMovieOpen)
         fileCloseMovie(pMovieInfo, TRUE);
     
     /* we have a .AVI movie to open, use MCI */
     /* set up the open parameters */
     
     mciOpen.dwCallback = NULL;
     mciOpen.wDeviceID = mciOpen.wReserved0 =
         mciOpen.wReserved1 = 0;
     mciOpen.lpstrDeviceType = NULL;
     mciOpen.lpstrElementName = szFilename;
     mciOpen.lpstrAlias = NULL;
     mciOpen.dwStyle = NULL;//WS_CHILD;
     mciOpen.hWndParent = NULL;//pMovieInfo->hwndParent;

     /* try to open the file */
     
     if (mciSendCommand(0, MCI_OPEN, 
         (DWORD)(MCI_OPEN_ELEMENT|MCI_DGV_OPEN_PARENT|MCI_DGV_OPEN_WS), 
       (DWORD)(LPMCI_DGV_OPEN_PARMS)&mciOpen) == 0){

         /* we opened the file o.k., now set up to */
         /* play it.                   */
         pMovieInfo->wMCIDeviceID = mciOpen.wDeviceID;  /* save ID */
         pMovieInfo->fMovieOpen = TRUE; /* a movie was opened */

         /* show the playback window */
         mciWindow.dwCallback = NULL;
         mciWindow.hWnd = pMovieInfo->hwndParent;
         mciWindow.wReserved1 = mciWindow.wReserved2 = 0;
         mciWindow.nCmdShow = SW_SHOW;
         mciWindow.lpstrText = (LPSTR)NULL;
         mciSendCommand(pMovieInfo->wMCIDeviceID, MCI_WINDOW, 
             MCI_DGV_WINDOW_STATE | MCI_DGV_WINDOW_HWND, 
             (DWORD)(LPMCI_DGV_WINDOW_PARMS)&mciWindow);

         /* get the window handle */
         mciStatus.dwItem = MCI_DGV_STATUS_HWND;
         mciSendCommand(pMovieInfo->wMCIDeviceID, 
             MCI_STATUS, MCI_STATUS_ITEM,
             (DWORD)(LPMCI_STATUS_PARMS)&mciStatus);
         pMovieInfo->hwndMovie = pMovieInfo->hwndParent;//(HWND)mciStatus.dwReturn;
         
         /* get movie length */         
         pMovieInfo->dwMovieLength = GetMovieLength(pMovieInfo);         
         pMovieInfo->dwCurrentFrame = 0;
                  
         /* now get the movie centered */
//         positionMovie(pMovieInfo);
     } else {
        /* generic error for open */
        MessageBox(pMovieInfo->hwndParent, "Unable to open Movie", NULL, 
                  MB_ICONEXCLAMATION|MB_OK);
        pMovieInfo->fMovieOpen = FALSE;
     }
   }
   
   /* cause an update to occur */
   InvalidateRect(pMovieInfo->hwndParent, NULL, FALSE);
   UpdateWindow(pMovieInfo->hwndParent);
}

/*--------------------------------------------------------------+
| playMovie - play/pause the movie depending on the state   |
|       of the <fPlaying> flag.             |
|                               |
| This function sets the <fPlaying> flag appropriately when done|
|                               |
+--------------------------------------------------------------*/
void playMovie(PMOVIEINFO pMovieInfo, WORD wDirection)
{
   pMovieInfo->fPlaying = !pMovieInfo->fPlaying;    /* swap the play flag */
   if (wDirection == NULL)
       pMovieInfo->fPlaying = FALSE;    /* wDirection == NULL means PAUSE */

   /* play/pause the AVI movie */
   if (pMovieInfo->fPlaying){       
       MCI_DGV_PLAY_PARMS   mciPlay;
       DWORD dwFlags = MCI_NOTIFY;       
           
       /* init to play all */
       mciPlay.dwCallback = MAKELONG(pMovieInfo->hwndMovie,0);
       mciPlay.dwFrom = mciPlay.dwTo = 0;       
       if (wDirection == 2)
           dwFlags |= MCI_DGV_PLAY_REVERSE;
       
       mciSendCommand(pMovieInfo->wMCIDeviceID, MCI_PLAY, dwFlags,
                       (DWORD)(LPMCI_DGV_PLAY_PARMS)&mciPlay);
   } else {
       MCI_DGV_PAUSE_PARMS  mciPause;
       mciPause.dwCallback = MAKELONG(pMovieInfo->hwndMovie,0);
                        
       /* tell it to pause */
       mciSendCommand(pMovieInfo->wMCIDeviceID,MCI_PAUSE,MCI_NOTIFY,
                     (DWORD)(LPMCI_DGV_PAUSE_PARMS)&mciPause);

   }
}

/*******************************************************************************************
    FUNCTION:   setMovie(PMOVIEINFO, DWORD, HWND)
    
    PURPOSE:    sets movie to specified frame;
                hCallback is not used currently (notice MCI_WAIT flag)
                it could easily be changed to MCI_CALLBACK
*******************************************************************************************/
void setMovie(PMOVIEINFO pMovieInfo, DWORD dwFrame, HWND hCallback)
{   
    MCI_SEEK_PARMS mciSeek;
    
    if (pMovieInfo->fPlaying){
                        playMovie(pMovieInfo, NULL);   
                        }                    
    
    mciSeek.dwTo = dwFrame;
    mciSeek.dwCallback = MAKELONG(pMovieInfo->hwndMovie,0);
    mciSendCommand(pMovieInfo->wMCIDeviceID, MCI_SEEK, MCI_TO | MCI_WAIT,
                        (DWORD)(LPMCI_SEEK_PARMS)&mciSeek);                    
}

/*--------------------------------------------------------------+
| seekMovie - seek in the movie depending on the wAction.   |
|         Possible actions are IDM_HOME (start of movie) or |
|         IDM_END (end of movie)                |
|                               |
|         Always stop the play before seeking.      |
|                               |
+--------------------------------------------------------------*/
void seekMovie(PMOVIEINFO pMovieInfo, WORD wAction)
{
   /* first stop the movie from playing if it is playing */
   if (pMovieInfo->fPlaying){
       playMovie(pMovieInfo, NULL);   
   }
   if (wAction == 0){
       /* home the movie */
       mciSendCommand(pMovieInfo->wMCIDeviceID, MCI_SEEK, MCI_SEEK_TO_START, 
            (DWORD)(LPVOID)NULL);
            
   } else if (wAction == 1){
       /* go to the end of the movie */
       mciSendCommand(pMovieInfo->wMCIDeviceID, MCI_SEEK, MCI_SEEK_TO_END, 
            (DWORD)(LPVOID)NULL);
   } 
}

/*--------------------------------------------------------------+
| stepMovie - step forward or reverse in the movie.  wDirection |
|       can be IDM_STEP (forward) or IDM_RSTEP (reverse)|
|                               |
|       Again, stop the play if one is in progress. |
|                               |
+--------------------------------------------------------------*/
void stepMovie(PMOVIEINFO pMovieInfo, HWND hWnd, WORD wDirection)
{
   MCI_DGV_STEP_PARMS   mciStep;
   
   if (pMovieInfo->fPlaying)
       playMovie(pMovieInfo, NULL);  /* turn off the movie */
   
       
   mciStep.dwFrames = 1L;
   if (wDirection == MCI_STEP)
       mciSendCommand(pMovieInfo->wMCIDeviceID, MCI_STEP, MCI_DGV_STEP_FRAMES,
           (DWORD)(LPMCI_DGV_STEP_PARMS)&mciStep);
   else
       mciSendCommand(pMovieInfo->wMCIDeviceID, MCI_STEP, 
           MCI_DGV_STEP_FRAMES|MCI_DGV_STEP_REVERSE,
           (DWORD)(LPMCI_DGV_STEP_PARMS)&mciStep);
}
