////////////////////////////////////////////////////////////////
// MFX Microsoft Foundation Class Library Extensions
// Copyright 1993 Paul DiLascia.
// 
// Implementation for CCapWnd, encalsulation of VFW capture window

#include "stdvid.h"

///////
// Declaration for private MFC functions (from afxpriv.h)
//
void PASCAL _AfxHookWindowCreate(CWnd* pWnd);
BOOL PASCAL _AfxUnhookWindowCreate();

IMPLEMENT_DYNAMIC(CCapWnd, CWnd)

BEGIN_MESSAGE_MAP(CCapWnd, CWnd)
	//{{AFX_MSG_MAP(CCapWnd)
	ON_WM_CREATE()
	ON_WM_DESTROY()
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

//////////////////
// Construct capture window: turn all callbacks off
//
CCapWnd::CCapWnd()
{

}

CCapWnd::~CCapWnd()
{
	if (m_hWnd != NULL)
		DestroyWindow();
}

//////////////////
// Create capture window. 
// Call VFW function to create, then attach myself.
//
BOOL CCapWnd::Create(LPCSTR lpszTitle, DWORD dwStyle, 
	int nID, CRect& rc, CWnd* pWndParent)
{
	_AfxHookWindowCreate(this);
	HWND hWnd = capCreateCaptureWindow(lpszTitle,
		dwStyle,
		rc.left, rc.top, rc.Width(), rc.Height(),
		pWndParent->GetSafeHwnd(),
		nID);
	_AfxUnhookWindowCreate();

	if (hWnd == NULL)
		return FALSE;
	ASSERT(hWnd == m_hWnd);
	return TRUE;
}

//////////////////
// Return place to hold original window proc
//
WNDPROC* CCapWnd::GetSuperWndProcAddr()
{
	static WNDPROC NEAR pfnSuper;
	return &pfnSuper;
}

//////////////////
// Do initialization when window is created.
// Install callbacks.
//
int CCapWnd::OnCreate(LPCREATESTRUCT lpcs)
{
	if (CWnd::OnCreate(lpcs) == -1)
		return -1;

	// Set callback procs
	SendMessage(WM_CAP_SET_CALLBACK_YIELD,			0, (LPARAM)CBYield);
	SendMessage(WM_CAP_SET_CALLBACK_STATUS,		0, (LPARAM)CBStatus);
	SendMessage(WM_CAP_SET_CALLBACK_ERROR,			0, (LPARAM)CBError);
	SendMessage(WM_CAP_SET_CALLBACK_FRAME,			0, (LPARAM)CBFrame);
	SendMessage(WM_CAP_SET_CALLBACK_VIDEOSTREAM,	0, (LPARAM)CBVideoStream);
	SendMessage(WM_CAP_SET_CALLBACK_WAVESTREAM,	0, (LPARAM)CBWaveStream);

	return 0; // ok
}

//////////////////
// Window is being destroyed: deinstall callbacks
//
void CCapWnd::OnDestroy()
{
	SendMessage(WM_CAP_SET_CALLBACK_ERROR,			0, 0L);
	SendMessage(WM_CAP_SET_CALLBACK_STATUS,		0, 0L);
	SendMessage(WM_CAP_SET_CALLBACK_YIELD,			0, 0L);
	SendMessage(WM_CAP_SET_CALLBACK_FRAME,			0, 0L);
	SendMessage(WM_CAP_SET_CALLBACK_VIDEOSTREAM,	0, 0L);
	SendMessage(WM_CAP_SET_CALLBACK_WAVESTREAM,	0, 0L);
	CWnd::OnDestroy();
}

////////////////////////////////////////////////////////////////
// Convert callbacks to virtual functions for standard capwin callbacks
//

CAPCALLBACK CCapWnd::CBYield(HWND hwnd)
{
	CCapWnd* pCapWin = (CCapWnd*)FromHandlePermanent(hwnd);
	ASSERT(pCapWin && pCapWin->ISA(CCapWnd));
	return pCapWin->OnYield();
}

LRESULT CCapWnd::OnYield()
{
	return TRUE;						// continue streaming
}

CAPCALLBACK CCapWnd::CBStatus(HWND hwnd, int id, LPSTR msg)
{
	CCapWnd* pCapWin = (CCapWnd*)FromHandlePermanent(hwnd);
	ASSERT(pCapWin && pCapWin->ISA(CCapWnd));
	return pCapWin->OnStatusChanged(id, msg);
}

LRESULT CCapWnd::OnStatusChanged(int /*id*/, LPSTR /* msg*/)
{
	return TRUE;
}

CAPCALLBACK CCapWnd::CBError(HWND hwnd, int id, LPSTR msg)
{
	CCapWnd* pCapWin = (CCapWnd*)FromHandlePermanent(hwnd);
	ASSERT(pCapWin && pCapWin->ISA(CCapWnd));
	return pCapWin->OnError(id, msg);
}

LRESULT CCapWnd::OnError(int /*id*/, LPSTR /*msg*/)
{
	return TRUE;
}

CAPCALLBACK CCapWnd::CBFrame(HWND hwnd, LPVIDEOHDR lpVH)
{
	CCapWnd* pCapWin = (CCapWnd*)FromHandlePermanent(hwnd);
	ASSERT(pCapWin && pCapWin->ISA(CCapWnd));
	return pCapWin->OnFrame(lpVH);
}

LRESULT CCapWnd::OnFrame(LPVIDEOHDR /* lpVH */)
{
	return TRUE;
}

CAPCALLBACK CCapWnd::CBVideoStream(HWND hwnd, LPVIDEOHDR lpVH)
{
	CCapWnd* pCapWin = (CCapWnd*)FromHandlePermanent(hwnd);
	ASSERT(pCapWin && pCapWin->ISA(CCapWnd));
	return pCapWin->OnVideoStream(lpVH);
}

LRESULT CCapWnd::OnVideoStream(LPVIDEOHDR /* lpVH */)
{
	return TRUE;
}

CAPCALLBACK CCapWnd::CBWaveStream(HWND hwnd, LPWAVEHDR lpWH)
{
	CCapWnd* pCapWin = (CCapWnd*)FromHandlePermanent(hwnd);
	ASSERT(pCapWin && pCapWin->ISA(CCapWnd));
	return pCapWin->OnWaveStream(lpWH);
}

LRESULT CCapWnd::OnWaveStream(LPWAVEHDR /* lpWH */)
{
	return TRUE;
}

////////////////////////////////////////////////////////////////
// Utility functions to get names in CStrings instead of char* bufs
//

CString CCapWnd::DriverGetName()
{ 
	char temp[256];
	return CString(DriverGetName(temp, sizeof(temp)) ? temp : "");
}

CString CCapWnd::DriverGetVersion()
{
	char temp[256];
	return CString(DriverGetVersion(temp, sizeof(temp)) ? temp : "");
}

CString CCapWnd::FileGetCaptureFile()
{
	char temp[256];
	return CString(FileGetCaptureFile(temp, sizeof(temp)) ? temp : "");
}

CString CCapWnd::GetMCIDeviceName()
{
	char temp[256];
	return CString(GetMCIDeviceName(temp, sizeof(temp)) ? temp : "");
}
