////////////////////////////////////////////////////////////////
// MFXVID: MFC Video Extensions
// Copyright 1994 Paul DiLascia.
//
// Implementation for CMciWnd, encalsulation of VFW MCIWnd class

#include "stdvid.h"

IMPLEMENT_DYNAMIC(CMciWnd, CWnd)

BEGIN_MESSAGE_MAP(CMciWnd, CWnd)
	//{{AFX_MSG_MAP(CMciWnd)
	ON_MESSAGE(WM_DRAWITEM,		OnOwnerDrawItem)
	ON_MESSAGE(WM_MEASUREITEM, OnOwnerDrawItem)
	ON_MESSAGE(WM_DELETEITEM,	OnOwnerDrawItem)
	ON_MESSAGE(WM_COMPAREITEM, OnOwnerDrawItem)
	ON_WM_DESTROY()
	ON_WM_CREATE()
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

///////
// Declaration for private MFC functions (from afxpriv.h)
//
void PASCAL _AfxHookWindowCreate(CWnd* pWnd);
BOOL PASCAL _AfxUnhookWindowCreate();

//////////////////
// Construct capture window: turn all callbacks off
//
CMciWnd::CMciWnd()
{
	// nothing to do
}

CMciWnd::~CMciWnd()
{
	// nothing to do
}

//////////////////
// There's really no need for this in CMciWnd except to demonstrate
// whether or not WM_CREATE is in fact trapped, which only happens if
// _AfxHookWindowCreate is used in the Create function.
//
int CMciWnd::OnCreate(LPCREATESTRUCT lpCreateStruct)
{
	TRACE0("CMciWnd::OnCreate\n");
	if (CWnd::OnCreate(lpCreateStruct) == -1)
		return -1;
	return 0;
}

//////////////////
// Create MCI window. 
// Call VFW function to create, then attach myself.
//
BOOL CMciWnd::Create(CWnd* pWndParent, DWORD dwStyle, LPCSTR szFile)
{
#ifndef __NOAFXHOOK

	// To see what happens without using create hooks, just 
	// #define __NOAFXHOOK 
	// And notice that CMciWnd::OnCreate is never called.
	// Using AFX hooks guarantees that WM_CREATE is trapped.
	// 
	_AfxHookWindowCreate(this);
HWND hWnd;
//	HWND hWnd = MCIWndCreate (pWndParent->GetSafeHwnd(),
//		AfxGetInstanceHandle(),
//		dwStyle,
//		(LPSTR)szFile);
	if (!_AfxUnhookWindowCreate())
		PostNcDestroy();

	if (hWnd == NULL)
		return FALSE;

#else

	// Create the "normal" way, using SubclassWindow to attach.
	// WM_CREATE is not trapped.
	//
	HWND hWnd = MCIWndCreate(pWndParent->GetSafeHwnd(),
		AfxGetInstanceHandle(),
		dwStyle,
		(LPSTR)szFile);

	if (hWnd == NULL)
		return FALSE;

	// Attach myself to HWND
	SubclassWindow(hWnd);

#endif

	ASSERT(hWnd == m_hWnd);
	return TRUE;
}

//////////////////
// Return place to hold original window proc
//
WNDPROC* CMciWnd::GetSuperWndProcAddr()
{
	static WNDPROC NEAR pfnSuper;
	return &pfnSuper;
}

//////////////////
// WM_COMMAND handler must be overridden to avoid the ASSERTs in MFC.
// MCIWnd uses non-standard parameters for WM_COMMAND, so I trap
// them and pass to the default message proc.
//
BOOL CMciWnd::OnCommand(WPARAM wParam, LPARAM lParam)
{
	UINT nID = LOWORD(wParam);
	HWND hWndCtrl = (HWND)LOWORD(lParam);

	Default();

	if (nID != 0 && (hWndCtrl==NULL || ::IsWindow(hWndCtrl)))
		CWnd::OnCommand(wParam, lParam);

	return TRUE;
}

//////////////////
// MCIWnd handles all the owner-draw messages itself, the old "C" way,
// so don't send these messages through the fancy MFC system.
// 
LRESULT CMciWnd::OnOwnerDrawItem(WPARAM /* wp */, LPARAM /* lp */)
{
	return Default();
}

//////////////////
// Make sure the device is really closed.
//
LONG CMciWnd::Close()
{
	UINT uDevice = GetDeviceID();

	SendMessage(MCI_CLOSE, MCI_WAIT, 0);
	if (uDevice>0) {
		//////
		// SendMessage(MCI_CLOSE) doesn't seem to close the device
		DWORD dwErr = mciSendCommand(uDevice, MCI_CLOSE, MCI_WAIT, 0);
		if (dwErr!=0)
			TRACE2("MCI_CLOSE device=%u, err = %lu\n", uDevice, dwErr);
		return dwErr;
	}
	return 0;
}

//////////////////
// Close the device when the window is destroyed
//
void CMciWnd::OnDestroy()
{
	// Microsofties don't always close the MCI device properly!!
	Close();
	Default();
}


