////////////////////////////////////////////////////////////////
// MFXVID: MFC Video Extensions
// Copyright 1994 Paul DiLascia.
//
// MFX Video for Windows support library. 
// Definitions for CMciWnd
//

#ifndef _MFXVID_H
#define _MFXVID_H

#include "vfw.h"

//////////////////
// C++ Wrapper for VFW MciWnd class.
//
class CMciWnd : public CWnd {
private:
	DECLARE_DYNAMIC(CMciWnd)
	virtual WNDPROC* GetSuperWndProcAddr();

protected:
	BOOL m_bOpen;				// whether open

public:
	CMciWnd();
	~CMciWnd();

	BOOL IsOpen()	{ return m_bOpen; }
	BOOL Create(CWnd* pWndParent, DWORD dwStyle=0, LPCSTR szFile=NULL);

	virtual BOOL OnCommand(WPARAM wParam, LPARAM lParam);

	// Brainless wrappers for MCI functions
	LONG Save(LPCSTR szFile)
		{ return MCIWndSave(m_hWnd, szFile); }
	LONG SaveDialog() // use SendMessage to avoid annoying warning message
		{ return SendMessage(MCI_SAVE,0,-1); }
	LONG New(LPVOID lp)
		{ return MCIWndNew(m_hWnd, lp); }
	LONG Record()
		{ return MCIWndRecord(m_hWnd); }
	LONG Open(LPCSTR sz, UINT flags=0)
		{ return MCIWndOpen(m_hWnd, sz, flags); }
	LONG OpenDialog() // use SendMessage to avoid annoying warning message
		{ return SendMessage(MCI_OPEN,0,-1); }
	LONG Close();
	LONG Play()				{ return MCIWndPlay(m_hWnd); }
	LONG Stop()				{ return MCIWndStop(m_hWnd); }
	LONG Pause()			{ return MCIWndPause(m_hWnd); }
	LONG Resume()			{ return MCIWndResume(m_hWnd); }
	LONG Seek(LONG lPos)	{ return MCIWndSeek(m_hWnd, lPos); }
	LONG Eject()			{ return MCIWndEject(m_hWnd); }
	LONG Home()				{ return MCIWndHome(m_hWnd); }
	LONG End()				{ return MCIWndEnd(m_hWnd); }
	LONG PlayReverse()	{ return MCIWndPlayReverse(m_hWnd); }
	LONG PlayFrom(LONG lPos)
		{ return MCIWndPlayFrom(m_hWnd, lPos); }
	LONG PlayTo(LONG lPos)
		{ return MCIWndPlayTo(m_hWnd, lPos); }
	LONG PlayFromTo(LONG lStart, LONG lEnd)
		{ return MCIWndPlayFromTo(m_hWnd, lStart, lEnd); }
	UINT GetDeviceID()
		{ return MCIWndGetDeviceID(m_hWnd); }
	UINT GetAlias()
		{ return MCIWndGetAlias(m_hWnd); }
	LONG GetMode(LPSTR lpsz=NULL, UINT len=0)
		{ return MCIWndGetMode(m_hWnd, lpsz, len); }
	LONG GetPosition()	{ return MCIWndGetPosition(m_hWnd); }
	LONG GetPositionString(LPSTR lpsz, UINT len)
		{ return MCIWndGetPositionString(m_hWnd, lpsz, len); }
	LONG GetStart()		{ return MCIWndGetStart(m_hWnd); }
	LONG GetLength()		{ return MCIWndGetLength(m_hWnd); }
	LONG GetEnd()			{ return MCIWndGetEnd(m_hWnd); }
	LONG Step(LONG n)		{ return MCIWndStep(m_hWnd, n); }
	LONG UseFrames()		{ return MCIWndUseFrames(m_hWnd); }
	LONG UseTime()			{ return MCIWndUseTime(m_hWnd); }
	LONG GetVolume()		{ return MCIWndGetVolume(m_hWnd); }
	LONG GetSpeed()		{ return MCIWndGetSpeed(m_hWnd); }
	BOOL GetRepeat()		{ return MCIWndGetRepeat(m_hWnd); }
	UINT GetZoom()			{ return MCIWndGetZoom(m_hWnd); }
	UINT GetActiveTimer()
		{ return MCIWndGetActiveTimer(m_hWnd); }
	UINT GetInactiveTimer()
		{ return MCIWndGetInactiveTimer(m_hWnd); }
	void SetZoom(UINT iZoom)
		{ MCIWndSetZoom(m_hWnd, iZoom); }
	LONG SetVolume(UINT iVol)
		{ return MCIWndSetVolume(m_hWnd, iVol); }
	LONG SetSpeed(UINT iSpeed)
		{ return MCIWndSetSpeed(m_hWnd, iSpeed); }
	LONG SetTimeFormat(LPSTR szFmt)
		{ return MCIWndSetTimeFormat(m_hWnd, szFmt); }
	LONG GetTimeFormat(LPSTR szFmt, UINT len)
		{ return MCIWndGetTimeFormat(m_hWnd, szFmt, len); }
	void ValidateMedia()	
		{ MCIWndValidateMedia(m_hWnd); }
	void SetRepeat(BOOL f)
		{ MCIWndSetRepeat(m_hWnd, f); }
	void SetActiveTimer(UINT msec)
		{ MCIWndSetActiveTimer(m_hWnd, msec); }
	void SetInactiveTimer(UINT msec)
		{ MCIWndSetInactiveTimer(m_hWnd, msec); }
	void SetTimers(UINT active, UINT inactive)
		{ MCIWndSetTimers(m_hWnd, active, inactive); }
	LONG Realize(BOOL fBkgnd)
		{ return MCIWndRealize(m_hWnd, fBkgnd); }
	LONG SendString(LPCSTR sz)
		{ return MCIWndSendString(m_hWnd, sz); }
	LONG ReturnString(LPVOID lpv, UINT len)
		{ return MCIWndReturnString(m_hWnd, lpv, len); }
	LONG GetError(LPSTR buf, UINT len)
		{ return MCIWndGetError(m_hWnd, buf, len); }
	HPALETTE GetPalette()
		{ return MCIWndGetPalette(m_hWnd); }
	LONG SetPalette(HPALETTE hpal)
		{ return MCIWndSetPalette(m_hWnd, hpal); }
	LONG GetFileName(LPSTR lp, UINT len)
		{ return MCIWndGetFileName(m_hWnd, lp, len); }
	LONG GetDevice(LPVOID lp, UINT len)
		{ return MCIWndGetDevice(m_hWnd, lp, len); }
	UINT GetStyles()
		{ return MCIWndGetStyles(m_hWnd); }
	LONG ChangeStyles(UINT mask, LONG val)
		{ return MCIWndChangeStyles(m_hWnd, mask, (LPARAM)val); }

protected:
	//{{AFX_MSG(CMciWnd)
	afx_msg LRESULT OnOwnerDrawItem(WPARAM wp, LPARAM lp);
	afx_msg void	 OnDestroy();
	afx_msg int		 OnCreate(LPCREATESTRUCT lpCreateStruct);
	//}}AFX_MSG
	DECLARE_MESSAGE_MAP()
};

#define CAPCALLBACK LRESULT CALLBACK AFX_EXPORT

//////////////////
// Bonus: CapWnd class encapsulates capture window. 
// Not described in the article (sorry, there wasn't enough space).
//
class CCapWnd : public CWnd {
private:
	DECLARE_DYNAMIC(CCapWnd)

	// Callback functions
	static CAPCALLBACK CBYield(HWND);
	static CAPCALLBACK CBStatus(HWND, int, LPSTR);
	static CAPCALLBACK CBError(HWND, int, LPSTR);
	static CAPCALLBACK CBFrame(HWND, LPVIDEOHDR);
	static CAPCALLBACK CBVideoStream(HWND, LPVIDEOHDR);
	static CAPCALLBACK CBWaveStream(HWND, LPWAVEHDR);

	virtual WNDPROC* GetSuperWndProcAddr();

public:
	CCapWnd();
	~CCapWnd();

	BOOL Create(LPCSTR pszTitle, DWORD dwStyle, 
		int id, CRect& rc, CWnd* pWndParent);

	// virtual fn implementation for callback message functions
	virtual LRESULT OnYield();
	virtual LRESULT OnStatusChanged(int id, LPSTR msg);
	virtual LRESULT OnError(int id, LPSTR msg);
	virtual LRESULT OnFrame(LPVIDEOHDR lpVH);
	virtual LRESULT OnVideoStream(LPVIDEOHDR lpVH);
	virtual LRESULT OnWaveStream(LPWAVEHDR lpWH);

	CString DriverGetName();
	CString DriverGetVersion();
	CString FileGetCaptureFile();
	CString GetMCIDeviceName();

	// Thin wrappers around AviCap window:
	BOOL DriverConnect(int iDriver=0)	
		{ return (BOOL)SendMessage(WM_CAP_DRIVER_CONNECT,(WPARAM)iDriver,0L); }
	BOOL DriverDisconnect()	
		{ return (BOOL)SendMessage(WM_CAP_DRIVER_DISCONNECT, (WPARAM)0, 0L); }
	BOOL DriverGetName(LPSTR szName, WORD wSize)
		{ return (BOOL)SendMessage(WM_CAP_DRIVER_GET_NAME, 
			(WPARAM)wSize, (LPARAM)(LPVOID)(LPSTR)szName); }
	BOOL DriverGetVersion(LPSTR szVer, WORD wSize)
		{ return (BOOL)SendMessage(WM_CAP_DRIVER_GET_VERSION, 
			wSize, (LPARAM)szVer); }
	BOOL DriverGetCaps(CAPDRIVERCAPS& caps)
		{ return (BOOL)SendMessage(WM_CAP_DRIVER_GET_CAPS, 
			sizeof(CAPDRIVERCAPS), (LPARAM)(LPVOID)&caps); }
	BOOL FileSetCaptureFile(LPCSTR szName)
		{ return (BOOL)SendMessage(WM_CAP_FILE_SET_CAPTURE_FILE, 
			0, (LPARAM)szName); }
	BOOL FileGetCaptureFile(LPSTR szName, WORD wSize)
		{ return (BOOL)SendMessage(WM_CAP_FILE_GET_CAPTURE_FILE,
			(WPARAM)wSize, (LPARAM)(LPVOID)(LPSTR)szName); }
	BOOL FileAlloc(DWORD dwSize)
		{ return (BOOL)SendMessage(WM_CAP_FILE_ALLOCATE, 0, (LPARAM)dwSize); }
	BOOL FileSaveAs(LPCSTR szName)
		{ return (BOOL)SendMessage(WM_CAP_FILE_SAVEAS, 0, (LPARAM)szName); }
	BOOL EditCopy()
		{ return (BOOL)SendMessage(WM_CAP_EDIT_COPY, 0, 0L); }
//	BOOL SetAudioFormat(WAVEFORMATEX &wf)
//		{ return (BOOL)SendMessage(WM_CAP_SET_AUDIOFORMAT,
//			(WPARAM)sizeof(WAVEFORMATEX), (LPARAM)&wf); }
//	DWORD GetAudioFormat(WAVEFORMATEX &wf)
//		{ return (DWORD)SendMessage(WM_CAP_GET_AUDIOFORMAT,
//			(WPARAM)sizeof(WAVEFORMATEX), (LPARAM)&wf); }
	DWORD GetAudioFormatSize()
		{ return (DWORD)SendMessage(WM_CAP_GET_AUDIOFORMAT, 0, 0L); }
	BOOL DlgVideoFormat()
		{ return (BOOL)SendMessage(WM_CAP_DLG_VIDEOFORMAT, 0, 0L); }
	BOOL DlgVideoSource()
		{ return (BOOL)SendMessage(WM_CAP_DLG_VIDEOSOURCE, 0, 0L); }
	BOOL DlgVideoDisplay()
		{ return (BOOL)SendMessage(WM_CAP_DLG_VIDEODISPLAY, 0, 0L); }
	BOOL DlgVideoCompression()
		{ return (BOOL)SendMessage(WM_CAP_DLG_VIDEOCOMPRESSION, 0, 0L); }
	DWORD GetVideoFormat(LPBITMAPINFOHEADER lpBmi, WORD wSize)
		{ return (DWORD)SendMessage(WM_CAP_GET_VIDEOFORMAT,
			wSize, (LPARAM)lpBmi); }
	DWORD GetVideoFormatSize()
		{ return (DWORD)SendMessage(WM_CAP_GET_VIDEOFORMAT, 0, 0L); }
	BOOL SetVideoFormat(LPBITMAPINFOHEADER lpBmi, WORD wSize)
		{ return (BOOL)SendMessage(WM_CAP_SET_VIDEOFORMAT,
			wSize, (LPARAM)lpBmi); }
	BOOL Preview(BOOL f)
		{ return (BOOL)SendMessage(WM_CAP_SET_PREVIEW, f, 0L); }
	BOOL PreviewRate(WORD wMilliSec)
		{ return (BOOL)SendMessage(WM_CAP_SET_PREVIEWRATE, wMilliSec, 0); }
	BOOL Overlay(BOOL f)
		{ return (BOOL)SendMessage(WM_CAP_SET_OVERLAY, f, 0L); }
	BOOL PreviewScale(BOOL f)
		{ return (BOOL)SendMessage(WM_CAP_SET_SCALE, f, 0L); }
	BOOL GetStatus(CAPSTATUS &status)
		{ return (BOOL)SendMessage(WM_CAP_GET_STATUS, 
			sizeof(CAPSTATUS), (LPARAM)&status); }
	BOOL SetScrollPos(POINT p)
		{ return (BOOL)SendMessage(WM_CAP_SET_SCROLL, 0, (LPARAM)(LPPOINT)&p); }
	BOOL GrabFrame()
		{ return (BOOL)SendMessage(WM_CAP_GRAB_FRAME, 0, 0L); }
	BOOL GrabFrameNoStop()
		{ return (BOOL)SendMessage(WM_CAP_GRAB_FRAME_NOSTOP, 0, 0L); }
	BOOL CaptureSequence()
		{ return (BOOL)SendMessage(WM_CAP_SEQUENCE, 0, 0L); }
	BOOL CaptureSequenceNoFile()
		{ return (BOOL)SendMessage(WM_CAP_SEQUENCE_NOFILE, 0, 0L); }
	BOOL CaptureStop()
		{ return (BOOL)SendMessage(WM_CAP_STOP, 0, 0L); }
	BOOL CaptureAbort()
		{ return (BOOL)SendMessage(WM_CAP_ABORT, 0, 0L); }
	BOOL CaptureSingleFrameOpen()
		{ return (BOOL)SendMessage(WM_CAP_SINGLE_FRAME_OPEN, 0, 0L); }
	BOOL CaptureSingleFrameClose()
		{ return (BOOL)SendMessage(WM_CAP_SINGLE_FRAME_CLOSE, 0, 0L); }
	BOOL CaptureSingleFrame()
		{ return (BOOL)SendMessage(WM_CAP_SINGLE_FRAME, 0, 0L); }
	BOOL CaptureGetSetup(CAPTUREPARMS &cp)
		{ return (BOOL)SendMessage(WM_CAP_GET_SEQUENCE_SETUP,
			sizeof(CAPTUREPARMS), (LPARAM)(LPVOID)&cp); }
	BOOL CaptureSetSetup(CAPTUREPARMS &cp)
		{ return (BOOL)SendMessage(WM_CAP_SET_SEQUENCE_SETUP,
			sizeof(CAPTUREPARMS), (LPARAM)&cp); }
	BOOL SetMCIDeviceName(LPCSTR szName)
		{ return (BOOL)SendMessage(WM_CAP_SET_MCI_DEVICE, 0, (LPARAM)szName); }
	BOOL GetMCIDeviceName(LPSTR szName, WORD wSize)
		{ return (BOOL)SendMessage(WM_CAP_GET_MCI_DEVICE,wSize,(LPARAM)szName);}
	BOOL PaletteOpen(LPCSTR szName)
		{ return (BOOL)SendMessage(WM_CAP_PAL_OPEN, 0, (LPARAM)szName); }
	BOOL PaletteSave(LPCSTR szName)
		{ return (BOOL)SendMessage(WM_CAP_PAL_SAVE, 0, (LPARAM)szName); }
	BOOL PalettePaste()
		{ return (BOOL)SendMessage(WM_CAP_PAL_PASTE, 0, 0L); }
	BOOL PaletteAuto(int iFrames, int iColors)
		{ return (BOOL)SendMessage(WM_CAP_PAL_AUTOCREATE,
			iFrames,(LPARAM)iColors); }
	BOOL PaletteManual(BOOL fGrab, int iColors)
		{ return (BOOL)SendMessage(WM_CAP_PAL_MANUALCREATE, 
			fGrab, (LPARAM)iColors); }

protected:
	//{{AFX_MSG(CCapWnd)
	afx_msg int OnCreate(LPCREATESTRUCT lpCreateStruct);
	afx_msg void OnDestroy();
	//}}AFX_MSG
	DECLARE_MESSAGE_MAP()
};


#endif // _MFXVID_H
