////////////////
// PLAYER.CPP Copyright 1994 MSJ Written by Paul DiLascia
// Multimedia AVI player using CMciWnd class.
//

#include "stdafx.h"
#include "player.h"

#ifdef _DEBUG
#undef THIS_FILE
static char BASED_CODE THIS_FILE[] = __FILE__;
#endif

//////////////////
// Frame window based on CMciWnd
//
class CMciFrame : public CMciWnd {
	BOOL m_fInitMenu;			// whether menu initialized or not
public:
	CMciFrame();

	virtual void PostNcDestroy();	
	virtual BOOL OnCmdMsg(UINT nID, int nCode, void* pExtra,
		AFX_CMDHANDLERINFO* pHandlerInfo);

	afx_msg void OnNcDestroy();
	afx_msg void OnInitMenu(CMenu* pMenu);

	DECLARE_MESSAGE_MAP()
};

BEGIN_MESSAGE_MAP(CMciFrame, CMciWnd)
	ON_WM_NCDESTROY()
	ON_WM_INITMENU()
END_MESSAGE_MAP()

CMciFrame::CMciFrame()
{
	m_fInitMenu = FALSE;
}

//////////////////
// When frame is destroyed, post message to quit application.
// Mimics normal frame window behavior.
//
void CMciFrame::OnNcDestroy()
{
	if (AfxGetApp()->m_pMainWnd == this) 
		PostQuitMessage(0); 		// Bye bye
	CMciWnd::OnNcDestroy();
}

////////////////
// (This is exactly like the MFC class CFrameWnd.)
// Default for frame windows is to allocate them on the heap;
// the default post-cleanup is to 'delete this'.
// Never call 'delete' on a CMciFrame, use DestroyWindow instead.
//
void CMciFrame::PostNcDestroy()
{
	delete this;
}

//////////////////
// Initialize main menu: Add "Exit" command.
// "View" is the name of the menu, even though you can't see it.
//
void CMciFrame::OnInitMenu(CMenu* pMenu)
{
	Default();
	if (pMenu==NULL || m_fInitMenu)
		return;

	char temp[40];
	pMenu->GetMenuString(0, temp, sizeof(temp), MF_BYPOSITION);
	if (strncmp(temp,"&View",5)==0) {
		pMenu->AppendMenu(MF_SEPARATOR);
		pMenu->AppendMenu(0, ID_APP_EXIT, "E&xit\tAlt+F4");
		m_fInitMenu = TRUE;
	}
}

//////////////////
// CMciFrame command/message routing.
// Do like a normal frame: pump unhandled messages through app.
//
BOOL CMciFrame::OnCmdMsg(UINT nID, int nCode, void* pExtra,
	AFX_CMDHANDLERINFO* pHandlerInfo)
{
	// pump through frame
	if (CWnd::OnCmdMsg(nID, nCode, pExtra, pHandlerInfo))
		return TRUE;

	// pump through app
	CWinApp* pApp = AfxGetApp();
	if (pApp != NULL &&
	  pApp->OnCmdMsg(nID, nCode, pExtra, pHandlerInfo))
		return TRUE;

	return FALSE;
}

/////////////////////////////////////////////////////////////////////////////
// CPlayerApp
//
CPlayerApp::CPlayerApp()
{
	// nothing to do
}

//////////////////
// The one and only CPlayerApp object
//
CPlayerApp NEAR theApp;

//////////////////
// CPlayerApp initialization
//
BOOL CPlayerApp::InitInstance()
{
	// Create a player frame and install it as the main window
	//
	CMciFrame * pPlayFrame = new CMciFrame;
	ASSERT(pPlayFrame);
	pPlayFrame->Create(NULL, 
		MCIWNDF_SHOWNAME,
		m_lpCmdLine[0] ? m_lpCmdLine : NULL);
	m_pMainWnd = pPlayFrame;

	// If filename specified, play it.
	if (m_lpCmdLine[0]) {
		TRACE1("Playing file %s\n", m_lpCmdLine);
		pPlayFrame->Play();
	}
	return TRUE;
}

