//==========================================================================;
//
//  THIS CODE AND INFORMATION IS PROVIDED "AS IS" WITHOUT WARRANTY OF ANY
//  KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
//  IMPLIED WARRANTIES OF MERCHANTABILITY AND/OR FITNESS FOR A PARTICULAR
//  PURPOSE.
//
//  Copyright (c) 1994  Microsoft Corporation.  All Rights Reserved.
//
//--------------------------------------------------------------------------;
//
//  profile.h
//
//  Description:
//
//      This file contains definitions required to support common profiling
//      source code.  This code reads profiles from the specified .ini file
//      under Win16, and read from
//
//          HKEY_LOCAL_MACHINE\Software\Microsoft\Multimedia\<subkey>
//
//      for Win32.
//
//==========================================================================;

#ifndef _PROFILE_H_
#define _PROFILE_H_

#ifdef __cplusplus
extern "C"                          // assume C declarations for C++
{
#endif

#ifndef INLINE
    #define INLINE __inline
#endif


//
//  Unicode compatibility.
//
#ifndef WIN32

    #ifndef _TCHAR_DEFINED
        #define _TCHAR_DEFINED
        typedef char            TCHAR, *PTCHAR;
        typedef unsigned char   TBYTE, *PTUCHAR;

        typedef PSTR            PTSTR, PTCH;
        typedef LPSTR           LPTSTR, LPTCH;
        typedef LPCSTR          LPCTSTR;
    #endif

    #define TEXT(a)         a
    #define SIZEOF(x)       sizeof(x)

#else

    #ifdef UNICODE
        #define SIZEOF(x)   (sizeof(x)/sizeof(WCHAR))
    #else
        #define SIZEOF(x)   sizeof(x)
    #endif

#endif



#ifdef WIN32

    //- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - ; 
    //
    //  WIN 32
    //
    //- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - ;

    UINT mmGetProfileInt
    (
        LPCTSTR lpszSection,
        LPCTSTR lpszEntry,
        UINT    uDefault
    );


    INLINE UINT mmGetPrivateProfileInt
    (
        LPCTSTR lpszSection,
        LPCTSTR lpszEntry,
        UINT    uDefault,
        LPCTSTR lpszFilename
    )
    {
        return mmGetProfileInt( lpszSection, lpszEntry, uDefault );
    }


    DWORD mmGetProfileString
    (
        LPCTSTR lpszSection,
        LPCTSTR lpszEntry,
        LPCTSTR lpszDefault,
        LPTSTR  lpszReturnBuffer,
        DWORD   cchReturnBuffer
    );
       

    INLINE DWORD mmGetPrivateProfileString
    (
        LPCTSTR lpszSection,
        LPCTSTR lpszEntry,
        LPCTSTR lpszDefault,
        LPTSTR  lpszReturnBuffer,
        DWORD   cchReturnBuffer,
        LPCTSTR lpszFilename
    )
    {
        return mmGetProfileString( lpszSection, lpszEntry, lpszDefault,
                        lpszReturnBuffer, cchReturnBuffer );
    }


    BOOL mmWriteProfileInt
    (
        LPCTSTR lpszSection,
        LPCTSTR lpszEntry,
        UINT    uOut
    );


    INLINE BOOL mmWritePrivateProfileInt
    (
        LPCTSTR lpszSection,
        LPCTSTR lpszEntry,
        UINT    uOut,
        LPCTSTR lpszFilename
    )
    {
        return mmWriteProfileInt( lpszSection, lpszEntry, uOut );
    }
    

    BOOL mmWriteProfileString
    (
        LPCTSTR lpszSection,
        LPCTSTR lpszEntry,
        LPCTSTR lpszString
    );

       
    INLINE BOOL mmWritePrivateProfileString
    (
        LPCTSTR lpszSection,
        LPCTSTR lpszEntry,
        LPCTSTR lpszString,
        LPCTSTR lpszFilename
    )
    {
        return mmWriteProfileString( lpszSection, lpszEntry, lpszString );
    }

#else


    //- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - ; 
    //
    //  WIN 16
    //
    //- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - ;


    INLINE UINT mmGetProfileInt
    (
        LPCSTR  lpszSection,
        LPCSTR  lpszEntry,
        UINT    uDefault
    )
    {
        return GetProfileInt( lpszSection, lpszEntry, (int)uDefault );
    }


    INLINE UINT mmGetPrivateProfileInt
    (
        LPCSTR  lpszSection,
        LPCSTR  lpszEntry,
        UINT    uDefault,
        LPCSTR  lpszFilename
    )
    {
        return GetPrivateProfileInt( lpszSection, lpszEntry, (int)uDefault,
                                                            lpszFilename );
    }


    INLINE DWORD mmGetProfileString
    (
        LPCSTR  lpszSection,
        LPCSTR  lpszEntry,
        LPCSTR  lpszDefault,
        LPSTR   lpszReturnBuffer,
        DWORD   cchReturnBuffer
    )
    {
        return (DWORD)GetProfileString( lpszSection, lpszEntry, lpszDefault,
                    lpszReturnBuffer, (int)cchReturnBuffer );
    }
       

    INLINE int mmGetPrivateProfileString
    (
        LPCSTR  lpszSection,
        LPCSTR  lpszEntry,
        LPCSTR  lpszDefault,
        LPSTR   lpszReturnBuffer,
        DWORD   cchReturnBuffer,
        LPCSTR  lpszFilename
    )
    {
        return (int)GetPrivateProfileString( lpszSection, lpszEntry,
                    lpszDefault, lpszReturnBuffer, (int)cchReturnBuffer,
                    lpszFilename );
    }


    BOOL mmWriteProfileInt
    (
        LPCSTR  lpszSection,
        LPCSTR  lpszEntry,
        UINT    uOut
    );


    BOOL mmWritePrivateProfileInt
    (
        LPCSTR  lpszSection,
        LPCSTR  lpszEntry,
        UINT    uOut,
        LPCSTR  lpszFilename
    );
    

    INLINE BOOL mmWriteProfileString
    (
        LPCSTR  lpszSection,
        LPCSTR  lpszEntry,
        LPCSTR  lpszString
    )
    {
        return WriteProfileString( lpszSection, lpszEntry, lpszString );
    }

       
    INLINE BOOL mmWritePrivateProfileString
    (
        LPCSTR  lpszSection,
        LPCSTR  lpszEntry,
        LPCSTR  lpszString,
        LPCSTR  lpszFilename
    )
    {
        return WritePrivateProfileString( lpszSection, lpszEntry, lpszString,
                                                            lpszFilename );
    }
    
   
#endif  // Win32


#endif  // _PROFILE_H_
