////////////////////////////////////////////////////////////////
// VIDDLG.CPP Copyright 1994 MSJ Written by Paul DiLascia
//
// Demonstrates use of video window (CMciWnd) in a dialog box.
//

#include "stdafx.h"
#include "viddlg.h"

#include "mainfrm.h"

#ifdef _DEBUG
#undef THIS_FILE
static char BASED_CODE THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CVidDlgApp
//
BEGIN_MESSAGE_MAP(CVidDlgApp, CWinApp)
	ON_COMMAND(ID_APP_ABOUT,	OnAppAbout)
	ON_COMMAND(ID_FILE_NEW,		CWinApp::OnFileNew)
	ON_COMMAND(ID_FILE_OPEN,	CWinApp::OnFileOpen)
END_MESSAGE_MAP()

/////////////////
// This function is described in the article.
//
BOOL MfxDestroyStaticAndGetRect(CWnd *pParentWnd, int id, CRect& rc);

/////////////////////////////////////////////////////////////////////////////
// CVidDlgApp
//
CVidDlgApp::CVidDlgApp()
{
	// nothing to do
}

////////////////
// The one and only CVidDlgApp object
//
CVidDlgApp NEAR theApp;

////////////////
// CVidDlgApp initialization
//
BOOL CVidDlgApp::InitInstance()
{
	// VIDDLG doesn't use documents or views, so just create a frame window
	//
   CMainFrame* pMainFrame = new CMainFrame;
   if (!pMainFrame->LoadFrame(IDR_MAINFRAME))
      return FALSE;
   pMainFrame->ShowWindow(m_nCmdShow);
   pMainFrame->UpdateWindow();
   m_pMainWnd = pMainFrame;

	return TRUE;
}

////////////////////////////////////////////////////////////////
// CMciControl is a control derived from CMciWnd.
//
// Mouse function is overloaded so clicking 
// inside the control plays the video.
//
class CMciControl : public CMciWnd {
protected:
	//{{AFX_MSG(CMciControl)
	afx_msg void OnLButtonDown(UINT nFlags, CPoint point);
	//}}AFX_MSG
	DECLARE_MESSAGE_MAP()
};

BEGIN_MESSAGE_MAP(CMciControl, CMciWnd)
	//{{AFX_MSG_MAP(CMciControl)
	ON_WM_LBUTTONDOWN()
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

void CMciControl::OnLButtonDown(UINT, CPoint)
{
	Play();	// play the video
}

//////////////////
// CAboutDlg dialog used for the About command
//
class CAboutDlg : public CDialog
{
	CMciControl	m_wndMci;	// video control
public:
	CAboutDlg();
	virtual BOOL OnInitDialog();
protected:
	//{{AFX_MSG(CAboutDlg)
	afx_msg LRESULT OnMciNotify(WPARAM wp, LPARAM lp);
	//}}AFX_MSG
	DECLARE_MESSAGE_MAP()
};

BEGIN_MESSAGE_MAP(CAboutDlg, CDialog)
	//{{AFX_MSG_MAP(CAboutDlg)
	ON_MESSAGE(MCIWNDM_NOTIFYMODE,	OnMciNotify)
	ON_MESSAGE(MCIWNDM_NOTIFYPOS,		OnMciNotify)
	ON_MESSAGE(MCIWNDM_NOTIFYSIZE,	OnMciNotify)
	ON_MESSAGE(MCIWNDM_NOTIFYMEDIA,	OnMciNotify)
	ON_MESSAGE(MCIWNDM_NOTIFYERROR,	OnMciNotify)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

CAboutDlg::CAboutDlg() : CDialog(IDD_ABOUTBOX)
{
	// nothing to do
}

//////////////////
// Initialize dialog: Replace static control with MCIWnd.
//
BOOL CAboutDlg::OnInitDialog()
{
	BOOL bRet = CDialog::OnInitDialog();

	CFile f;
	if ( f.Open("VIDDLG.AVI", CFile::modeRead) ) {
		CRect rc;
		MfxDestroyStaticAndGetRect(this, IDC_STATICAVI, rc);

		m_wndMci.Create(this, 
			MCIWNDF_NOAUTOSIZEWINDOW |		// we will set size
			MCIWNDF_NOPLAYBAR |				// no play/pause buttons or slider
			MCIWNDF_NOMENU |					// no popup menu from RBUTTONDOWN
			MCIWNDF_NOTIFYALL,				// notify me of stuff
			"VIDDLG.AVI");

		m_wndMci.MoveWindow(&rc, FALSE);
		m_wndMci.PostMessage(MCI_PLAY);
	}

	return bRet;
}

//////////////////
// I've never gotten this message
//
LRESULT CAboutDlg::OnMciNotify(WPARAM wp, LPARAM lp)
{
	extern MSG NEAR _afxLastMsg;	// naughty me, peeking inside MFC

	const char* MsgTable[] = {	"MCIWNDM_NOTIFYMODE",
		"MCIWNDM_NOTIFYPOS",
		"MCIWNDM_NOTIFYSIZE",
		"MCIWNDM_NOTIFYMEDIA",
		"MCIWNDM_NOTIFYERROR"
	};

	TRACE1("Got MCI notification: %s\n",
		MsgTable[_afxLastMsg.message-MCIWNDM_NOTIFYMODE]);
	return Default();
}

////////////////
// Run the About dialog
//
void CVidDlgApp::OnAppAbout()
{
	CAboutDlg aboutDlg;
	aboutDlg.DoModal();
}

//////////////////
// This function destroys a static control and gets its 
// window rectangle in parent window coordinates.
//
BOOL MfxDestroyStaticAndGetRect(CWnd *pParentWnd, int id, CRect& rc)
{
	ASSERT(pParentWnd);
	CWnd *pChild = pParentWnd->GetDlgItem(id);

	// Debug: check that child exists and is a static
	if (pChild==NULL) {
		TRACE1("MFX: Child %d not found\n", id);
		return FALSE;

	} else if (!(pChild->SendMessage(WM_GETDLGCODE, 0, 0L) & DLGC_STATIC)) {
		TRACE1("MFX: Control %d is not a static\n", id);
	}

	// Found child and it is a static
	pChild->GetWindowRect(&rc);
	pParentWnd->ScreenToClient(&rc);		// convert to parent coords
	pChild->DestroyWindow();				// and destroy the child
	return TRUE;
}
