
/******************************************************************************\
*       This is a part of the Microsoft Source Code Samples. 
*       Copyright (C) 1994 Microsoft Corporation.
*       All rights reserved. 
*       This source code is only intended as a supplement to 
*       Microsoft Development Tools and/or WinHelp documentation.
*       See these sources for detailed information regarding the 
*       Microsoft samples programs.
\******************************************************************************/

/*************************************************************************\
*  PROGRAM: npclnt16.c
*
*  PURPOSE:
*
*     To demonstrate Windows for Workgroups' named pipe APIs.
*     This code serves as the client side of the named pipe instances.
*     For more details on an overview of this codes designs or use, see
*     the README file.  For details on the implementation, see the comments
*     in this code.
*
*  Functions:
*     WinMain()            - Entrypoint for application
*     InitDlgProc()        - Get connection info for named pipe
*     ClientDlgProc()      - The main dialog procedure
*     AsyncReadCallback()  - Callback procedure for async reading
*
*  Written by Microsoft Product Support Services, Windows Developer Support  
\*************************************************************************/

#define  STRICT
#include <windows.h>
#include "npclnt16.h"
#include <string.h>
#include <stdio.h>
#include <stdlib.h>
#include <neterr.H>
#include <nmpipe.h>

/*************************************************************************\
*  GLOBALS
\*************************************************************************/
HINSTANCE hInst;
HWND   hWndClient;
char szServer[LINE_LEN];      
char szClient[NAME_SIZE];     

/*************************************************************************\
*
*  FUNCTION: WinMain(HANDLE, HANDLE, LPSTR, int)
*
*  PURPOSE: Launches the Client's dialog box.
*
*
\*************************************************************************/
int PASCAL   WinMain (HINSTANCE hInstance,
                      HINSTANCE hPrevInstance,
                      LPSTR     lpCmdLine,
                      int       nCmdShow)


{
   DWORD retCode;
   FARPROC lpDlgProc;

   hInst   = hInstance;

   lpDlgProc = MakeProcInstance((FARPROC) ClientDlgProc, hInst );
   retCode = DialogBox (hInst, (LPSTR)"ClientDialog",
                        NULL, (DLGPROC)lpDlgProc);
   FreeProcInstance(lpDlgProc);

   return  ((int)retCode);

   UNREFERENCED_PARAMETER( nCmdShow );
   UNREFERENCED_PARAMETER( lpCmdLine );
   UNREFERENCED_PARAMETER( hPrevInstance );
   
}


/*************************************************************************\
*
*  PROCEDURE: InitDlgProc (HWND hDlg, WORD wMsg, LONG wParam, LONG lParam)
*
*  PURPOSE:  This dialog box prompts the user for a server name and
*            a client or user name.  These values are placed into global
*            strings szServer and szClient.
*
*  CALLED BY:
*
*    ClientDlgProc();
*
\*************************************************************************/
LONG CALLBACK InitDlgProc (HWND hDlg, UINT wMsg, WPARAM wParam, LPARAM lParam)
  {

   UNREFERENCED_PARAMETER(lParam);

   switch (wMsg)
      {
      case WM_INITDIALOG:  // set text limits
         PostMessage (GetDlgItem (hDlg, IDD_SVREDIT),  
                      EM_LIMITTEXT, LINE_LEN, 0);
         PostMessage (GetDlgItem (hDlg, IDD_CLNTEDIT),
                      EM_LIMITTEXT, NAME_SIZE, 0);

      case WM_COMMAND:
         switch (LOWORD(wParam)) 
            {                            // When the user clicks okay, get the
            case IDB_INITOK:             // share name and user name from the
                                         // edit fields.
               GetWindowText (GetDlgItem (hDlg, IDD_SVREDIT),
                              (LPSTR)szServer, LINE_LEN);
               GetWindowText (GetDlgItem (hDlg, IDD_CLNTEDIT),
                              (LPSTR)szClient, NAME_SIZE);
               EndDialog(hDlg, 0);
               return (0);
               break;

            default:
               return (0);
            }
      default:
         return (0);
      }
   return (0);
}


/*************************************************************************\
*
*  PROCEDURE: ClientDlgProc (HWND hDlg, WORD wMsg, LONG wParam, LONG lParam)
*
*  PURPOSE:  This procedure services the dialog box that serves as an interface
*            to the named pipe server instance.  The larger edit field is used
*            to read messages from the server instance.  The smaller edit field
*            is used to type messages to the server instance.  This procedure
*            is responsible for connecting to the named pipe, creating a
*            seperate thread to read the pipe, and sending to the pipe.
*
*  CALLED BY:
*
*    WinMain();
*
*  CALLS TO:
*
*    InitDlgProc();
*
\**************************************************************************/

LONG CALLBACK ClientDlgProc (HWND hDlg, UINT wMsg, WPARAM wParam, LPARAM lParam)
{
   char   szPipe[LINE_LEN+NAME_SIZE+2];
   char   szBuf[OUT_BUF_SIZE];
   unsigned short uRet;
   FARPROC lpDlgProc = NULL;
   int    ndx;

   static char     szWriteBuf[OUT_BUF_SIZE];
   static char     szReadBuf[IN_BUF_SIZE+2];  // allow space for newlines
   static unsigned short cbRead;
   static unsigned short uReadErr;

   static HFILE hFile = NULL;
   static FARPROC lpReadProc  = NULL; 
   static BOOL bReading = FALSE;
   static BOOL bClose = FALSE;

   switch (wMsg) {

      case WM_COMMAND:
         switch (LOWORD(wParam)) {

         // When the user presses Send: capture the string from the edit
         // field, prepend it with the user name, write it to the server
         // instance of the named pipe.

            case IDB_SEND:                 // Get the text from the edit field.
               GetWindowText (GetDlgItem(hDlg,IDD_EDITWRITE),
                              szBuf, OUT_BUF_SIZE);

                                          // Prepend it with the user name
               wsprintf ((LPSTR)szWriteBuf, "%s: %s",
                         (LPSTR)szClient, (LPSTR)szBuf);
          
                                       // Write the client name to server.
               if ( ( _lwrite( hFile, szWriteBuf, strlen(szWriteBuf)+1 ) )
                                                         == HFILE_ERROR ) {
                  MessageBox (hDlg,
                              "Cannot write to pipe.",
                              szClient,
                              MB_OK);
                  PostMessage(hDlg, WM_GO_AWAY, 0, 0L );
                  }
               SetFocus( GetDlgItem(hDlg, IDD_EDITWRITE) );
               return TRUE;
               break;

            case IDD_EDITREAD:
               if ( HIWORD( lParam ) == EN_SETFOCUS ) {
                  SetFocus( GetDlgItem(hDlg, IDD_EDITWRITE) );
                  return TRUE;
                  }
               return FALSE;
               break;

            default:
               return (0);
            }
         break;

      case WM_ASYNC_READ:
         bReading = FALSE; // signal that the read has returned

         if ( uReadErr == NERR_Success ) {
                                       // Write message to larger edit field.
            szReadBuf[cbRead] = '\0';
            lstrcat( (LPSTR)szReadBuf, (LPSTR)"\r\n" );

            ndx = GetWindowTextLength(GetDlgItem (hDlg, IDD_EDITREAD));

            SendMessage (GetDlgItem (hDlg, IDD_EDITREAD),
                         EM_SETSEL, 0, MAKELONG( ndx, ndx ) );

            SendMessage (GetDlgItem (hDlg, IDD_EDITREAD),
                         EM_REPLACESEL,
                         0, (LPARAM)((LPSTR)szReadBuf));

            uReadErr = 0;              // start next async read
            cbRead = 0; 
            uRet = DosReadAsyncNmPipe(hFile, (API_CALLBACK)lpReadProc,
                                      (unsigned short FAR *) &uReadErr,
                                      (unsigned char FAR *) szReadBuf,
                                      IN_BUF_SIZE,
                                      (unsigned short FAR *) &cbRead );

            if ( uRet != 0 ) {         // if call was unsuccessfull - close
               wsprintf( (LPSTR)szBuf, (LPSTR)"Error Reading Pipe: %u", uRet );
               MessageBox(hDlg, szBuf, szClient, MB_OK );
               PostMessage( hDlg, WM_SYSCOMMAND, SC_CLOSE, 0L );
               }
            else
               bReading = TRUE;        // success - signal wait for read
            }
         else {                        // async read returned with error
            if ( !bClose ) {           // if error not caused by close
               wsprintf( (LPSTR)szBuf, //   display message
                         (LPSTR)"Async Pipe Read Error: %u", uRet );
               MessageBox(hDlg, szBuf, szClient, MB_OK );
               }
            PostMessage( hDlg, WM_SYSCOMMAND, SC_CLOSE, 0L );
            }
         return TRUE;
         break;



      case WM_INITCLIENT:
       // On initialization, use the Init dialog box prompt the user for a
       // server name and a client or user name. Named pipe names should have
       // the form '\\<machinename>\PIPE\<pipename>' for remote machines.
       // Once the server name is captured from the Init dialog box,
       // convert the name into the proper form; then do a _lopen()
       // to connect to the pipe.  Next write the user name to the server
       // of the named pipe.  Finally, start first async read

                                       // Launch Init dialog box to capture
                                       // share name and user name.
       lpDlgProc = MakeProcInstance((FARPROC) InitDlgProc, hInst );
       DialogBox (hInst,
                  (LPSTR)"InitDialog",
                  (HWND)hDlg,
                  (DLGPROC)lpDlgProc);
       FreeProcInstance(lpDlgProc);

                                       // Put captured user name in window
                                       // caption.
       SetWindowText (hDlg, szClient);

                                       // Construct file/pipe name.
       wsprintf ((LPSTR)szPipe, "\\\\%s\\pipe\\test", (LPSTR)szServer);

                                       // try openning pipe
       if ( (hFile = _lopen(szPipe, READ_WRITE)) == HFILE_ERROR ) {
         MessageBox (hDlg,
                     "CANNOT OPEN PIPE: Assure Server32 is started, check server name.",
                     szClient,
                     MB_OK);
         EndDialog (hDlg, 0);          // Kill app if pipe didn't connect.
         return 0;
         };

                                       // Write the client name to server.
       if ( ( _lwrite( hFile, szClient, strlen(szClient) ) ) == HFILE_ERROR ) {
         MessageBox (hDlg,
                     "Cannot write to pipe.",
                     szClient,
                     MB_OK);
         _lclose( hFile );
         EndDialog (hDlg, 0);
         return 0;
         }

       lpReadProc  = MakeProcInstance((FARPROC) AsyncReadCallback, hInst );

       uReadErr = 0;
       cbRead = 0;
       uRet = DosReadAsyncNmPipe(hFile, (API_CALLBACK)lpReadProc,(unsigned short FAR *) &uReadErr ,(unsigned char FAR *) szReadBuf, IN_BUF_SIZE,(unsigned short FAR *) &cbRead );
       if ( uRet != 0 ) {
          wsprintf( szBuf, "Error Reading Pipe: %u", uRet );
          MessageBox(hDlg, szBuf, szClient, MB_OK );
          PostMessage( hDlg, WM_SYSCOMMAND, SC_CLOSE, 0L );
          }
       else 
          bReading = TRUE;

       return TRUE;


     case WM_INITDIALOG:               // PostMessage() give time for the                         
       hWndClient = hDlg;              // dialog box to be created.
       PostMessage (hDlg, WM_INITCLIENT, 0, 0);
       SetFocus(GetDlgItem(hDlg,IDD_EDITWRITE));
       return (0);
       break;


     case WM_SYSCOMMAND:
       if (wParam == SC_CLOSE)
         {
         if ( bReading ) {  // allow async read callback to close dialog
            bClose = TRUE;  // after pipe is closed
            _lclose( hFile );
            }
         else {
            _lclose( hFile );
            FreeProcInstance( lpReadProc );
            EndDialog( hDlg, 0 );
            }
         return TRUE;
         }
       break;
     }
    return (FALSE);

}



/*************************************************************************\
*
*  PROCEDURE: AsyncReadCallback( unsigned char FAR * pbBuffer )
*
*  PURPOSE: To notify client app the asyncronous read has returned
*
*
\**************************************************************************/
void FAR PASCAL AsyncReadCallback(unsigned char FAR * pbBuffer )
{
PostMessage( hWndClient, WM_ASYNC_READ, 0, 0 );
}
